/***********************************************************************
 * Linotte                                                             *
 * Version release date : October 26, 2005                             *
 * Author : Mounes Ronan ronan.mounes@amstrad.eu                       *
 *                                                                     *
 *     http://langagelinotte.free.fr                                   *
 *                                                                     *
 * This code is released under the GNU GPL license, version 2 or       *
 * later, for educational and non-commercial purposes only.            *
 * If any part of the code is to be included in a commercial           *
 * software, please contact us first for a clearance at                *
 *   ronan.mounes@amstrad.eu                                           *
 *                                                                     *
 *   This notice must remain intact in all copies of this code.        *
 *   This code is distributed WITHOUT ANY WARRANTY OF ANY KIND.        *
 *   The GNU GPL license can be found at :                             *
 *           http://www.gnu.org/copyleft/gpl.html                      *
 *                                                                     *
 ***********************************************************************/

package org.linotte.frame.atelier;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagLayout;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.border.Border;

import org.linotte.frame.Atelier;
import org.linotte.greffons.GreffonsChargeur;
import org.linotte.greffons.api.Greffon;
import org.linotte.moteur.entites.Prototype;
import org.linotte.moteur.outils.FichierOutils;
import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.Version;

/**
 * Affichage de la boite A Propos
 *
 */

@SuppressWarnings("serial")
public class Apropos extends JDialog {

	private JLabel apropos;
	private String logo;
	private String COPYRIGHT = "\u00a9";
	private JTabbedPane tabbedPane;
	private JTextArea jTextArea;
	private Atelier parent;

	public Apropos(Atelier parent) {
		super(parent, "A propos de l'Atelier de programmation Linotte", true);
		setIconImage(Ressources.getImageIcon("help-browser.png").getImage());
		setLayout(new GridBagLayout());
		logo = Ressources.getURL("jeprogramme.png").toString();
		this.parent = parent;

		// Onglet
		tabbedPane = new JTabbedPane();
		JPanel panel = new JPanel();
		panel.setLayout(new GridBagLayout());

		apropos = new JLabel();
		apropos.setAlignmentX(Component.CENTER_ALIGNMENT);
		panel.add(apropos);

		tabbedPane.addTab("A propos", panel);

		jTextArea = new JTextArea();
		jTextArea.setEditable(false);
		jTextArea.setFont(Font.getFont(Font.SANS_SERIF));
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setViewportView(jTextArea);
		scrollPane.setPreferredSize(new Dimension(400, 400));
		tabbedPane.addTab("Nouveautés", scrollPane);

		getContentPane().add(tabbedPane);
		Border border = BorderFactory.createMatteBorder(10, 10, 10, 10, getBackground());
		apropos.setBorder(border);

		setDefaultCloseOperation(HIDE_ON_CLOSE);
		setResizable(false);

		construireTexte();
	}

	/**
	 * 
	 */
	private void construireTexte() {
		String greffons = "";
		List<Prototype> prototypes = new ArrayList<Prototype>();
		for (Prototype e : parent.linotte.especeModeleMap) {
			if (e.getGreffon() != null) {
				prototypes.add(e);
			}
		}

		Collections.sort(prototypes, new Comparator<Prototype>() {
			@Override
			public int compare(Prototype o1, Prototype o2) {
				return o1.getNom().toString().compareTo(o2.getNom().toString());
			}
		});

		int boucle = 1, taille = prototypes.size();
		if (taille > 0)
			for (Prototype e : prototypes) {

				Greffon greffon = GreffonsChargeur.getInstance().getGreffon(e.getNom().toString());

				greffons += greffon.getNom() + " " + greffon.getVersion();

				if (boucle != taille)
					greffons += ", ";
				if (boucle % 5 == 0) // On saute une ligne tous les 5 greffons 
					greffons += "<br>";

				boucle++;
			}
		else
			greffons = "aucun";

		String langageLinotte = Merci.htmlGras("<font color=blue>Langage L</font>") + "<small>&nbsp;inotte</small>";

		apropos.setText("<html>"
				+ Merci.htmlCentre(Merci.htmlGras("<img src=\"" + logo + "\" ><br>" + "&nbsp;" + langageLinotte
						+ "<br><font color=blue>L</font> comme littéraire,<br><font color=blue>L</font> comme libre !<br>" + Version.getVersion() + "<br/>"
						+ COPYRIGHT + " 2005-2015 Ronan Mounès"))

				+ "<br><br>"

				+ Merci.htmlGras("Greffons chargés :<br>") + greffons + "<br><br><table border=0><tr><td>" + Merci.htmlGras("Licence : ") + " GPL version 3"
				+ Merci.htmlGras("<br>Java : ") + System.getProperty("java.version") + "</td></tr></table>");
		pack();
	}

	@Override
	public void setVisible(boolean b) {
		if (b) {
			jTextArea.setText("");
			construireTexte();
		}
		this.setLocationRelativeTo(null);
		if (b) {
			// Il faut charger le texte après la fonction pack() !
			try {
				jTextArea.setText(FichierOutils.lire(new File(Ressources.getdirectory("."), "ALIRE.TXT")));
				jTextArea.setCaretPosition(0);
			} catch (Exception e1) {
				e1.printStackTrace();
				if (tabbedPane.getTabCount() > 1)
					tabbedPane.removeTabAt(1);
			}
		}
		super.setVisible(b);
	}
}