package org.linotte.greffons;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.linotte.frame.gui.SplashWindow;
import org.linotte.greffons.api.FabriqueGreffon;
import org.linotte.greffons.api.Greffon;
import org.linotte.greffons.java.GreffonPrototype;
import org.linotte.greffons.java.GreffonPrototype.Entrée;
import org.linotte.greffons.python.PythonFactory;
import org.linotte.moteur.outils.Preference;
import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.Version;

/**
 * @author CPC
 *
 */
public class GreffonsChargeur extends URLClassLoader implements FabriqueGreffon {

	// http://snippets.dzone.com/posts/show/3574

	private Map<String, Greffon> greffons = new HashMap<String, Greffon>();

	private static GreffonsChargeur chargeur;

	private static Map<String, FabriqueGreffon> fabriques = new HashMap<String, FabriqueGreffon>();

	static {
		try {
			SplashWindow.setProgressValue("Greffon : initialisation de JPython");
			fabriques.put("python", new PythonFactory());
		} catch (Throwable e) {
			if (Version.isBeta())
				e.printStackTrace();
		}
		try {
			//SplashWindow.setProgressValue("Greffon : initialisation de JRuby");
			//fabriques.put("ruby", new RubyFactory());
		} catch (Throwable e) {
			e.printStackTrace();
		}
		try {
			//SplashWindow.setProgressValue("Greffon : initialisation JSR223");
			//fabriques.put("jsr223", new Jsr223Factory());
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	private GreffonsChargeur(URL[] urls) {
		super(urls);
	}

	public List<Greffon> getGreffons() {
		return new ArrayList<Greffon>(greffons.values());
	}

	public Greffon getGreffon(String nom) {
		return greffons.get(nom);
	}

	public org.linotte.greffons.externe.Greffon newInstance(String id) {
		Greffon greffon = greffons.get(id);
		if (greffon != null) {
			return greffon.newInstance();
		} else
			return null;
	}

	public boolean ajouterJar(String jar) {
		jar = jar.replace("\\", "/");
		File fj = new File(jar);
		try {
			//System.out.println(fj.toString());
			addURL(fj.toURI().toURL());
		} catch (MalformedURLException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	public Greffon ajouterGreffon(String id, String classe, String lang) {
		FabriqueGreffon fabriqueGreffon = fabriques.get(lang);
		if (fabriqueGreffon == null) {
			if (lang != null) {
				String[] p = lang.split(":");
				if (p.length > 1 && p[0].equals("jsr223")) {
					fabriqueGreffon = fabriques.get("jsr223");
				}
			}
			if (fabriqueGreffon == null) {
				fabriqueGreffon = fabriques.get("java");
			}
		}
		if (greffons.get(id) != null) {
			System.out.println("Greffon déjà présent : " + id);
			return null;
		} else {
			Greffon greffon = fabriqueGreffon.produire(id, classe);
			greffons.put(id, greffon);
			return greffon;
		}
	}

	public static synchronized GreffonsChargeur getInstance() {
		if (chargeur == null) {
			try {
				// Prendre en compte les .class dans les repertoires greffons
				File repertoire_user = new File(Preference.getIntance().getHome() + File.separator + Preference.REPERTOIRE + File.separator
						+ GreffonsHandler.DIR_USER);
				URL urls[] = { repertoire_user.toURI().toURL(), Ressources.getGreffons().toURI().toURL() };
				for (URL url : urls) {
					if (Version.isBeta())
						System.out.println("Classpath java greffon : " + url);
				}
				chargeur = new GreffonsChargeur(urls);
			} catch (Exception e) {
				e.printStackTrace();
			}
			fabriques.put("java", chargeur);
		}
		return chargeur;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	public org.linotte.greffons.api.Greffon produire(String id, String classe) {
		try {
			Class c = loadClass(classe);
			try {
				Object o = c.newInstance();
				if (o instanceof org.linotte.greffons.externe.Greffon) {
					Greffon greffon = new Greffon();
					greffon.setClass_(c);
					greffon.setId(id);
					greffon.setObjet(o);
					// Ajout des méthodes fonctionnelles :
					List<Entrée> slots = GreffonPrototype.chargerSlotJava(greffon);
					greffon.setSlots(slots);
					return greffon;
				} else {
					return null;
				}
			} catch (InstantiationException e) {
				e.printStackTrace();
				return null;
			} catch (IllegalAccessException e) {
				e.printStackTrace();
				return null;
			}
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			return null;
		}
	}

}
